@@ -79,11 +79,18 @@
 #include "autosave.h"
 #include "dynamic_refresh.h"
 #include "dia_image.h"
+#include "persistence.h"
+#include "sheets.h"
 
 #if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
 extern DiaExportFilter png_export_filter;
 #endif
 
+static gboolean
+handle_initial_diagram(const char *input_file_name, 
+		       const char *export_file_name,
+		       const char *export_file_format);
+
 static void create_user_dirs(void);
 static PluginInitResult internal_plugin_init(PluginInfo *info);
 
@@ -145,14 +152,30 @@ build_output_file_name(const char *infname, const char *format)
 
 const char *argv0 = NULL;
 
-gboolean 
-do_convert(const char *infname,
-           const char *outfname)
+/** Convert infname to outfname, using input filter inf and export filter
+ * ef.  If either is null, try to guess them.
+ */
+gboolean
+do_convert(const char *infname, 
+	   const char *outfname, DiaExportFilter  *ef)
 {
-  DiaExportFilter *ef = NULL;
-  DiaImportFilter *inf = NULL;
+  DiaImportFilter *inf;
   DiagramData *diagdata = NULL;
 
+  inf = filter_guess_import_filter(infname);
+  if (!inf) 
+    inf = &dia_import_filter;
+
+  if (ef == NULL) {
+    ef = filter_guess_export_filter(outfname);
+    if (!ef) {
+      fprintf(stderr,
+	      _("%s error: don't know how to export into %s\n"),
+	      argv0,outfname);
+      exit(1);
+    }
+  }
+
   dia_is_interactive = FALSE;
 
   if (0==strcmp(infname,outfname)) {
@@ -163,22 +186,12 @@ do_convert(const char *infname,
   }
   
   diagdata = new_diagram_data(&prefs.new_diagram);
-  inf = filter_guess_import_filter(infname);
-  if (!inf) 
-    inf = &dia_import_filter;
   if (!inf->import(infname,diagdata,inf->user_data)) {
     fprintf(stderr,
             _("%s error: need valid input file %s\n"),
             argv0,infname);
             exit(1);
   }
-  ef = filter_guess_export_filter(outfname);
-  if (!ef) {
-    fprintf(stderr,
-            _("%s error: don't know how to export into %s\n"),
-            argv0,outfname);
-            exit(1);
-  }
   ef->export(diagdata, outfname, infname, ef->user_data);
   /* if (!quiet) */ fprintf(stdout,
                       _("%s --> %s\n"),
@@ -202,6 +215,53 @@ app_is_interactive(void)
   return dia_is_interactive;
 }
 
+/** Handle loading of diagrams given on command line, including conversions.
+ * Returns TRUE if any automatic conversions were performed.
+ */
+static gboolean
+handle_initial_diagram(const char *in_file_name, 
+		       const char *out_file_name,
+		       const char *export_file_format) {
+  DDisplay *ddisp = NULL;
+  Diagram *diagram = NULL;
+  gboolean made_conversions = FALSE;
+
+  if (export_file_format) {
+    char *export_file_name = NULL;
+    DiaExportFilter *ef;
+    /* First try guessing based on extension */
+    export_file_name = build_output_file_name(in_file_name,
+					      export_file_format);
+    ef = filter_guess_export_filter(export_file_name);
+    if (ef == NULL) {
+      ef = filter_get_by_name(export_file_format);
+      if (ef == NULL) {
+	g_error(_("Can't find output format %s\n"), export_file_format);
+	return FALSE;
+      }
+      export_file_name = build_output_file_name(in_file_name,
+						ef->extensions[0]);
+    }
+    made_conversions |= do_convert(in_file_name, (out_file_name != NULL?out_file_name:export_file_name), ef);
+    g_free(export_file_name);
+  } else if (out_file_name) {
+    made_conversions |= do_convert(in_file_name, out_file_name, NULL);
+  } else {
+    if (g_file_test(in_file_name, G_FILE_TEST_EXISTS)) {
+      diagram = diagram_load (in_file_name, NULL);
+    } else
+      diagram = new_diagram (in_file_name);
+	      
+    if (diagram != NULL) {
+      diagram_update_extents(diagram);
+      layer_dialog_set_diagram(diagram);
+                  
+      ddisp = new_display(diagram);
+    }
+  }
+  return made_conversions;
+}
+
 #ifdef G_OS_WIN32
 static void
 myXmlErrorReporting (void *ctx, const char* msg, ...)
@@ -218,6 +278,24 @@ myXmlErrorReporting (void *ctx, const char* msg, ...)
 }
 #endif
 
+#ifdef HAVE_FREETYPE
+#define EPS_PANGO "eps-pango, "
+#else
+#define EPS_PANGO ""
+#endif
+
+#ifdef G_OS_WIN32
+#define WMF "wmf, "
+#else
+#define WMF ""
+#endif
+
+#if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
+#define BITMAPS "png, "
+#else
+#define BITMAPS "bmp, gif, jpg, png, pnm, ras, tif, "
+#endif
+
 void
 app_init (int argc, char **argv)
 {
@@ -241,7 +319,12 @@ app_init (int argc, char **argv)
     {"export", 'e', POPT_ARG_STRING, NULL /* &export_file_name */, 0,
      N_("Export loaded file and exit"), N_("OUTPUT")},
     {"export-to-format",'t', POPT_ARG_STRING, NULL /* &export_file_format */,
-     0, N_("Export to file format and exit"), N_("eps,png,wmf,cgm,dxf,fig")},
+     0, N_("Export to file format and exit.  Supported formats are: "
+	   "cgm, dia, dxf, eps, " EPS_PANGO
+	   "fig, mp, plt, hpgl, " BITMAPS
+	   "shape, svg, tex, " WMF
+	   "wpg"), N_("FORMAT")
+    },
     {"nosplash", 'n', POPT_ARG_NONE, &nosplash, 0,
      N_("Don't show the splash screen"), NULL },
     {"credits", 'c', POPT_ARG_NONE, &credits, 0,
@@ -276,6 +359,7 @@ app_init (int argc, char **argv)
     GnomeProgram *program =
       gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
 			  argc, argv, GNOME_PARAM_POPT_TABLE, options,
+			  GNOME_PROGRAM_STANDARD_PROPERTIES,
 			  GNOME_PARAM_NONE);
     g_object_get(program, "popt-context", &poptCtx, NULL);
     client = gnome_master_client();
@@ -310,13 +394,6 @@ app_init (int argc, char **argv)
             exit(0);
         }
 
-        if (export_file_format && export_file_name) {
-            fprintf(stderr,
-                    _("%s error: can specify only one of -f or -o."),
-                    argv[0]);
-            exit(1);
-        }
-
     }
 #endif
 #endif
@@ -325,9 +402,9 @@ app_init (int argc, char **argv)
   if (version) {
 #if (defined __TIME__) && (defined __DATE__)
     /* TRANSLATOR: 2nd and 3rd %s are time and date respectively. */
-    printf(_("Dia version %s, compiled %s %s\n"), VERSION, __TIME__, __DATE__);
+    printf(g_locale_from_utf8(_("Dia version %s, compiled %s %s\n"), -1, NULL, NULL, NULL), VERSION, __TIME__, __DATE__);
 #else
-    printf(_("Dia version %s\n"), VERSION);
+    printf(g_locale_from_utf8(_("Dia version %s\n"), -1, NULL, NULL, NULL), VERSION);
 #endif
     exit(0);
   }
@@ -380,7 +457,6 @@ app_init (int argc, char **argv)
 
   dia_image_init();
 
-  
   gdk_rgb_init();
 
   gtk_rc_parse ("diagtkrc"); 
@@ -392,12 +468,6 @@ app_init (int argc, char **argv)
 
   color_init();
 
-#ifdef HAVE_FREETYPE
-  dia_font_init(pango_ft2_get_context(10,10));
-#else
-  dia_font_init(gdk_pango_context_get());
-#endif
-
   /* Init cursors: */
   default_cursor = gdk_cursor_new(GDK_LEFT_PTR);
   ddisplay_set_all_cursor(default_cursor);
@@ -423,13 +493,16 @@ app_init (int argc, char **argv)
 
   prefs_load();
 
-  create_layer_dialog();
+  persistence_load();
 
   /* further initialization *before* reading files */  
   active_tool = create_modify_tool();
 
   create_toolbox();
 
+  persistence_register_window_create("layer_window",
+				     (NullaryFunc*)&create_layer_dialog);
+
   /*fill recent file menu */
   recent_file_history_init();
 
@@ -438,40 +511,26 @@ app_init (int argc, char **argv)
 
   create_tree_window();
 
+  persistence_register_window_create("sheets_main_dialog",
+				     (NullaryFunc*)&sheets_dialog_create);
+
   /* In current setup, we can't find the autosaved files. */
   /*autosave_restore_documents();*/
   
   if (argv) {
 #ifdef HAVE_POPT
       while (poptPeekArg(poptCtx)) {
-          Diagram *diagram = NULL;
-          DDisplay *ddisp = NULL;
           char *in_file_name = (char *)poptGetArg(poptCtx);
-
-          if (export_file_name) {
-              made_conversions |= do_convert(in_file_name,export_file_name);
-          } else if (export_file_format) {
-              export_file_name = build_output_file_name(in_file_name,
-                                                        export_file_format);
-              made_conversions |= do_convert(in_file_name,export_file_name);
-              g_free(export_file_name);
-          } else {
-              diagram = diagram_load (in_file_name, NULL);
-              if (diagram != NULL) {
-                  diagram_update_extents(diagram);
-                  layer_dialog_set_diagram(diagram);
-                  
-                  ddisp = new_display(diagram);
-              }
-          }
+	  
+	  made_conversions |= handle_initial_diagram(in_file_name,
+						     export_file_name,
+						     export_file_format);
       }
       poptFreeContext(poptCtx);
 #else
       int i;
 
       for (i=1; i<argc; i++) {
-          Diagram *diagram = NULL;
-          DDisplay *ddisp;
           char *in_file_name = argv[i]; /* unless it's an option... */
           
           if (0==strcmp(argv[i],"-t")) {
@@ -488,24 +547,9 @@ app_init (int argc, char **argv)
               }
           }
           
-          if (export_file_name) {
-              made_conversions |= do_convert(in_file_name,export_file_name);
-          } else if (export_file_format) {
-              export_file_name = build_output_file_name(in_file_name,
-                                                        export_file_format);
-              made_conversions |= do_convert(in_file_name,export_file_name);
-              g_free(export_file_name);
-          } else {
-              diagram = diagram_load(in_file_name, NULL);
-              
-              if (diagram != NULL) {
-                  diagram_update_extents(diagram);
-                  layer_dialog_set_diagram(diagram);
-                  
-                  ddisp = new_display(diagram);
-              }
-              /* Error messages are done in diagram_load() */
-          }
+	  made_conversions |= handle_initial_diagram(in_file_name,
+						     export_file_name,
+						     export_file_format);
       }
 #endif
   }
@@ -533,7 +577,10 @@ app_exit(void)
    */
   static gboolean app_exit_once = FALSE;
 
-  g_return_if_fail (!app_exit_once);
+  if (app_exit_once) {
+    g_error(_("This shouldn't happen.  Please file a bug report at bugzilla.gnome.org\ndescribing how you can cause this message to appear.\n"));
+    return;
+  }
 
   if (diagram_modified_exists()) {
     GtkWidget *dialog;
@@ -566,6 +613,10 @@ app_exit(void)
       return;
   }
 
+  prefs_save();
+
+  persistence_save();
+
   dynobj_refresh_finish();
     
   dia_object_defaults_save (NULL);
@@ -596,6 +647,9 @@ app_exit(void)
   recent_file_history_write();
   
   gtk_main_quit();
+  /* This printf seems to prevent a race condition with unrefs. */
+  /* Yuck.  -Lars */
+  printf(_("Thank you for using Dia.\n"));
   app_exit_once = TRUE;
 }
 
@@ -651,9 +705,16 @@ internal_plugin_init(PluginInfo *info)
   filter_register_import(&dia_import_filter);
 
   /* register export filters */
+  /* Standard Dia format */
   filter_register_export(&dia_export_filter);
+  /* EPS with PS fonts */
   filter_register_export(&eps_export_filter);
+#ifdef HAVE_FREETYPE
+  /* EPS with Pango rendering */
+  filter_register_export(&eps_ft2_export_filter);
+#endif
 #if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
+  /* PNG with libart rendering */
   filter_register_export(&png_export_filter);
 #endif
 
