@@ -36,8 +36,9 @@ struct _Diagram {
   char *filename;
   int unsaved;            /* True if diagram is created but not saved.*/
   int modified;
-  int autosaved;          /* True if the diagram has been autosaved and not
-			     since modified */
+  gboolean autosaved;     /* True if the diagram is autosaved since last mod */
+  char *autosavefilename;     /* Holds the name of the current autosave file
+			       * for this diagram, or NULL.  */
   
   DiagramData *data;
   
@@ -47,7 +48,15 @@ struct _Diagram {
   UndoStack *undo;
 };
 
-extern GList *open_diagrams; /* Read only! */
+struct _object_extent
+{
+  Object *object;
+  Rectangle *extent;
+};
+
+typedef struct _object_extent object_extent;
+
+GList *dia_open_diagrams(void); /* Read only! */
 
 Diagram *diagram_load(const char *filename, DiaImportFilter *ifilter);
 int diagram_load_into (Diagram *dest, const char *filename, DiaImportFilter *ifilter);
@@ -78,12 +87,18 @@ void diagram_flush(Diagram *dia);
 Object *diagram_find_clicked_object(Diagram *dia,
 				    Point *pos,
 				    real maxdist);
+Object *diagram_find_clicked_object_except(Diagram *dia,
+					   Point *pos,
+					   real maxdist,
+					   GList *avoid);
 real diagram_find_closest_handle(Diagram *dia, Handle **handle,
 				 Object **obj, Point *pos);
 real diagram_find_closest_connectionpoint(Diagram *dia,
 					  ConnectionPoint **cp,
-					  Point *pos);
+					  Point *pos,
+					  Object *notthis);
 void diagram_update_extents(Diagram *dia);
+gint diagram_parent_sort_cb(object_extent ** a, object_extent **b);
 
 void diagram_update_menu_sensitivity (Diagram *dia, UpdatableMenuItems *items);
 void diagram_update_menubar_sensitivity(Diagram *dia, UpdatableMenuItems *items);
@@ -91,10 +106,16 @@ void diagram_update_popupmenu_sensitivity(Diagram *dia);
 
 void diagram_place_under_selected(Diagram *dia);
 void diagram_place_over_selected(Diagram *dia);
+void diagram_place_down_selected(Diagram *dia);
+void diagram_place_up_selected(Diagram *dia);
 void diagram_group_selected(Diagram *dia);
 void diagram_ungroup_selected(Diagram *dia);
+void diagram_parent_selected(Diagram *dia);
+void diagram_unparent_selected(Diagram *dia);
+void diagram_unparent_children_selected(Diagram *dia);
 
 void diagram_set_filename(Diagram *dia, char *filename);
+gchar *diagram_get_name(Diagram *dia);
 
 int diagram_modified_exists(void);
 
