@@ -41,6 +41,8 @@
 #include "intl.h"
 #include "magnify.h"
 #include "diamenu.h"
+#include "preferences.h"
+#include "scroll_tool.h"
 
 /* This contains the point that was clicked to get this menu */
 static Point object_menu_clicked_point;
@@ -200,9 +202,14 @@ popup_object_menu(DDisplay *ddisp, GdkEventButton *bevent)
       DiaMenuItem *item = &dia_menu->items[i];
       gtk_widget_set_sensitive(GTK_WIDGET(item->app_data),
 			       item->active & DIAMENU_ACTIVE);
-      if (item->active & DIAMENU_TOGGLE)
+      if (item->active & DIAMENU_TOGGLE) {
+	g_signal_handlers_block_by_func(GTK_CHECK_MENU_ITEM(item->app_data),
+					(GtkSignalFunc)object_menu_proxy, item);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item->app_data),
 				       item->active & DIAMENU_TOGGLE_ON);
+	g_signal_handlers_unblock_by_func(GTK_CHECK_MENU_ITEM(item->app_data),
+					  (GtkSignalFunc)object_menu_proxy, item);
+      }
     }
 
     menu = GTK_MENU(dia_menu->app_data);
@@ -421,10 +428,24 @@ ddisplay_canvas_events (GtkWidget *canvas,
         switch (sevent->direction)
         {
             case GDK_SCROLL_UP:
-              ddisplay_scroll_up(ddisp);
+              if (sevent->state & GDK_SHIFT_MASK)
+                  ddisplay_scroll_left(ddisp);
+              else if (sevent->state & GDK_CONTROL_MASK) {
+                  ddisplay_untransform_coords(ddisp, sevent->x, sevent->y, &middle.x, &middle.y);
+                  ddisplay_zoom(ddisp, &middle, 2);
+              }
+              else 
+                  ddisplay_scroll_up(ddisp);
               break;
             case GDK_SCROLL_DOWN:
-              ddisplay_scroll_down(ddisp);
+              if (sevent->state & GDK_SHIFT_MASK)
+                  ddisplay_scroll_right(ddisp);
+              else if (sevent->state & GDK_CONTROL_MASK) { 
+                    ddisplay_untransform_coords(ddisp, sevent->x, sevent->y, &middle.x, &middle.y);
+                    ddisplay_zoom(ddisp, &middle, 0.5);
+              }
+              else
+                  ddisplay_scroll_down(ddisp);
               break;
             case GDK_SCROLL_LEFT:
               ddisplay_scroll_left(ddisp);
@@ -472,6 +493,8 @@ ddisplay_canvas_events (GtkWidget *canvas,
         switch (bevent->button)
         {
             case 1:
+              if (transient_tool)
+                break;
               if (*active_tool->double_click_func)
                 (*active_tool->double_click_func) (active_tool, bevent, ddisp);
               break;
@@ -500,6 +523,8 @@ ddisplay_canvas_events (GtkWidget *canvas,
         switch (bevent->button)
         {
             case 1:
+              if (transient_tool)
+                break;
                   /* get the focus again, may be lost by zoom combo */
               gtk_widget_grab_focus(canvas);
               if (*active_tool->button_press_func)
@@ -510,9 +535,16 @@ ddisplay_canvas_events (GtkWidget *canvas,
               if (ddisp->menu_bar == NULL) {
                 popup_object_menu(ddisp, bevent);
               }
+	      else if (!transient_tool) {
+		gtk_widget_grab_focus(canvas);
+		transient_tool = create_scroll_tool();
+		(*transient_tool->button_press_func) (transient_tool, bevent, ddisp);
+	      }
               break;
 
             case 3:
+              if (transient_tool)
+                break;
               if (ddisp->menu_bar == NULL) {
                 if (bevent->state & GDK_CONTROL_MASK) {
                       /* for two button mouse users ... */
@@ -545,6 +577,13 @@ ddisplay_canvas_events (GtkWidget *canvas,
               break;
 
             case 2:
+	      if (transient_tool) {
+	        (*transient_tool->button_release_func) (transient_tool,
+  	                                             bevent, ddisp);
+								
+	        tool_free(transient_tool);
+	        transient_tool = NULL;
+	      }
               break;
 
             case 3:
@@ -568,7 +607,9 @@ ddisplay_canvas_events (GtkWidget *canvas,
           mevent->state = tmask;
           mevent->is_hint = FALSE;
         }
-        if (*active_tool->motion_func)
+        if (transient_tool && (*transient_tool->motion_func)) 
+          (*transient_tool->motion_func) (transient_tool, mevent, ddisp);
+        else if (*active_tool->motion_func)
           (*active_tool->motion_func) (active_tool, mevent, ddisp);
         break;
 
@@ -722,23 +763,125 @@ ddisplay_destroy (GtkWidget *widget, gpointer data)
   ddisplay_really_destroy(ddisp);
 }
 
-void
+inline int 
+round_up (double x)
+{
+  if (x - (int) x > 0.001)
+    return (int) x + 1;
+  else
+    return (int) x ;
+}
+
+
+/* returns NULL if object cannot be created */
+Object *
 ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
 		     gpointer user_data)
 {
   Point droppoint;
+  Point droppoint_orig;
   Handle *handle1, *handle2;
-  Object *obj;
+  Object *obj, *p_obj;
   GList *list;
+  real click_distance;
 
   ddisplay_untransform_coords(ddisp, x, y, &droppoint.x, &droppoint.y);
 
+  /* save it before snap_to_grid modifies it */
+  droppoint_orig = droppoint;
+
   snap_to_grid(ddisp, &droppoint.x, &droppoint.y);
 
   obj = dia_object_default_create (otype, &droppoint,
                                    user_data,
                                    &handle1, &handle2);
 
+
+  click_distance = ddisplay_untransform_length(ddisp, 3.0);
+
+  p_obj = diagram_find_clicked_object(ddisp->diagram, &droppoint_orig,
+				    click_distance);
+
+  if (p_obj && p_obj->can_parent) /* the tool was dropped inside an object that takes children*/
+  {
+    Rectangle *p_ext, *c_ext;
+    int new_height = 0, new_width = 0;
+    real parent_height, child_height, parent_width, child_width;
+    Point new_pos;
+
+    obj->parent = p_obj;
+    p_obj->children = g_list_append(p_obj->children, obj);
+
+    p_ext = parent_handle_extents(p_obj);
+    c_ext = parent_handle_extents(obj);
+
+    parent_height = p_ext->bottom - p_ext->top;
+    child_height = c_ext->bottom - c_ext->top;
+
+    parent_width = p_ext->right - p_ext->left;
+    child_width = c_ext->right - c_ext->left;
+
+    /* we need the pre-snap position */
+    c_ext->left = droppoint_orig.x;
+    c_ext->top = droppoint_orig.y;
+    c_ext->right = c_ext->left + child_width;
+    c_ext->bottom = c_ext->top + child_height;
+
+    /* check if the top of the child is inside the parent, but the bottom is not */
+    if (c_ext->top < p_ext->bottom && c_ext->bottom > p_ext->bottom)
+    {
+      /* check if child is smaller than parent height wise */
+      if (child_height < parent_height)
+        new_height = child_height;
+      /* check if parent is bigger than 1 in height */
+      else if (parent_height  > 1)
+        new_height = round_up(parent_height) - 1;
+    }
+    else
+    {
+      new_height = child_height;
+    }
+    /* check if the left of the child is inside the partent, but the right is not */
+    if (c_ext->left < p_ext->right && c_ext->right > p_ext->right)
+    {
+      /* check if child is smaller than parent width wise */
+      if (child_width < parent_width)
+        new_width = child_width;
+      /* check if parent is bigger than 1 in width */
+      else if (parent_width > 1)
+        new_width = round_up(parent_width) - 1;
+    }
+    else
+    {
+      new_width = child_width;
+    }
+
+    g_free(p_ext);
+    g_free(c_ext);
+
+    /* if we can't fit in both directions, produce an error */
+    if (!new_height && !new_width)
+    {
+      message_error(_("The object you dropped cannot fit into its parent. \nEither expand the parent object, or drop the object elsewhere."));
+      obj->parent->children = g_list_remove(obj->parent->children, obj);
+      obj->ops->destroy (obj);
+      return NULL;
+    }
+    /* if we can't fit height wise, make height same as of the parent */
+    else if (!new_height)
+      new_height = parent_height;
+    /* if we can't fit width wise, make the width same as of the parent */
+    else if (!new_width)
+      new_width = parent_width;
+
+    new_pos.x = droppoint.x + new_width;
+    new_pos.y = droppoint.y + new_height;
+    obj->ops->move_handle(obj, handle2, &new_pos, NULL,
+			  HANDLE_MOVE_USER, 0);
+
+  }
+
+
   diagram_add_object(ddisp->diagram, obj);
   diagram_remove_all_selected(ddisp->diagram, TRUE); /* unselect all */
   diagram_select(ddisp->diagram, obj);
@@ -750,6 +893,7 @@ ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
     object_connect_display(ddisp, obj, handle1);
   }
   object_add_updates(obj, ddisp->diagram);
+  ddisplay_do_update_menu_sensitivity(ddisp);
   diagram_flush(ddisp->diagram);
 
   list = g_list_prepend(NULL, obj);
@@ -758,4 +902,7 @@ ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
   diagram_modified(ddisp->diagram);
 
   undo_set_transactionpoint(ddisp->diagram->undo);
+  if (prefs.reset_tools_after_create)
+    tool_reset();
+  return obj;
 }
