@@ -71,17 +71,20 @@ static int get_handle_nr(PolyShape *poly, Handle *handle)
   return -1;
 }
 
-void
+ObjectChange*
 polyshape_move_handle(PolyShape *poly, Handle *handle,
-		     Point *to, HandleMoveReason reason)
+		      Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
   int handle_nr;
   
   handle_nr = get_handle_nr(poly, handle);
   poly->points[handle_nr] = *to;
+  
+  return NULL;
 }
 
-void
+ObjectChange*
 polyshape_move(PolyShape *poly, Point *to)
 {
   Point p;
@@ -94,6 +97,8 @@ polyshape_move(PolyShape *poly, Point *to)
   for (i=1;i<poly->numpoints;i++) {
     point_add(&poly->points[i], &p);
   }
+
+  return NULL;
 }
 
 int
@@ -243,24 +248,103 @@ polyshape_remove_point(PolyShape *poly, int pos)
 				old_cp1, old_cp2);
 }
 
+/** Returns the first clockwise direction in dirs 
+ * (as returned from find_slope_directions) */
+static gint
+first_direction(gint dirs) {
+  switch (dirs) {
+  case DIR_NORTHEAST: return DIR_NORTH;
+  case DIR_SOUTHEAST: return DIR_EAST;
+  case DIR_NORTHWEST: return DIR_WEST;
+  case DIR_SOUTHWEST: return DIR_SOUTH;
+  default: return dirs;
+  }
+}
+
+/** Returns the last clockwise direction in dirs 
+ * (as returned from find_slope_directions) */
+static gint
+last_direction(gint dirs) {
+  switch (dirs) {
+  case DIR_NORTHEAST: return DIR_EAST;
+  case DIR_SOUTHEAST: return DIR_SOUTH;
+  case DIR_NORTHWEST: return DIR_NORTH;
+  case DIR_SOUTHWEST: return DIR_WEST;
+  default: return dirs;
+  }
+}
+
+/** Returns the available directions for a corner */
+static gint 
+find_tip_directions(Point prev, Point this, Point next)
+{
+  gint startdirs = find_slope_directions(prev, this);
+  gint enddirs = find_slope_directions(this, next);
+  gint firstdir = first_direction(startdirs);
+  gint lastdir = last_direction(enddirs);
+  gint dir, dirs = 0;
+
+  dir = firstdir;
+  while (dir != lastdir) {
+    dirs |= dir;
+    dir = dir * 2;
+    if (dir == 16) dir = 1;
+  }
+  dirs |= dir;
+
+  return dirs;
+}
+
 void
 polyshape_update_data(PolyShape *poly)
 {
   Point next;
   int i;
+  Object *obj = &poly->object;
   
+  /* handle the case of whole points array update (via set_prop) */
+  if (poly->numpoints != obj->num_handles ||
+      poly->numpoints*2 != obj->num_connections) {
+    object_unconnect_all(obj); /* too drastic ? */
+
+    obj->handles = g_realloc(obj->handles, 
+                             poly->numpoints*sizeof(Handle *));
+    obj->num_handles = poly->numpoints;
+    for (i=0;i<poly->numpoints;i++) {
+      obj->handles[i] = g_new(Handle, 1);
+      setup_handle(obj->handles[i]);
+    }
+
+    obj->connections = g_realloc(obj->connections,
+                                 poly->numpoints * 2 * sizeof(ConnectionPoint *));
+    for (i = 0; i < poly->numpoints * 2; i++) {
+      obj->connections[i] = g_new0(ConnectionPoint, 1);
+      obj->connections[i]->object = obj;
+    }
+    poly->object.num_connections = poly->numpoints*2;
+  }
+
   /* Update handles: */
   for (i = 0; i < poly->numpoints; i++) {
+    gint thisdir, nextdir;
+    Point prev;
     poly->object.handles[i]->pos = poly->points[i];
 
-    poly->object.connections[2*i]->pos = poly->points[i];
+    prev = next;
     if (i == poly->numpoints - 1)
       next = poly->points[0];
     else
       next = poly->points[i+1];
     point_add(&next, &poly->points[i]);
     point_scale(&next, 0.5);
+
+    thisdir = find_tip_directions(prev, poly->points[i], next);
+    nextdir = find_slope_directions(poly->points[i], next);
+
+    poly->object.connections[2*i]->pos = poly->points[i];
+    poly->object.connections[2*i]->directions = thisdir;
     poly->object.connections[2*i+1]->pos = next;
+    poly->object.connections[2*i+1]->directions = nextdir;
   }
 }
 
