@@ -16,19 +16,20 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
 #include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "arrows.h"
 #include "properties.h"
@@ -51,11 +52,12 @@ struct _Participation {
 
 static real participation_distance_from(Participation *dep, Point *point);
 static void participation_select(Participation *dep, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void participation_move_handle(Participation *dep, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void participation_move(Participation *dep, Point *to);
-static void participation_draw(Participation *dep, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static ObjectChange* participation_move_handle(Participation *dep, Handle *handle,
+					       Point *to, ConnectionPoint *cp,
+					       HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* participation_move(Participation *dep, Point *to);
+static void participation_draw(Participation *dep, DiaRenderer *renderer);
 static Object *participation_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -69,9 +71,9 @@ static void participation_update_data(Participation *dep);
 static PropDescription *
 participation_describe_props(Participation *participation);
 static void
-participation_get_props(Participation *participation, Property *props, guint nprops);
+participation_get_props(Participation *participation, GPtrArray *props);
 static void
-participation_set_props(Participation *participation, Property *props, guint nprops);
+participation_set_props(Participation *participation, GPtrArray *props);
 static DiaMenu *participation_get_object_menu(Participation *participation,
 					      Point *clickedpoint);
 
@@ -108,7 +110,7 @@ static ObjectOps participation_ops = {
 };
 
 static PropDescription participation_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  ORTHCONN_COMMON_PROPERTIES,
   { "total", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
     N_("Total:"), NULL, NULL },
   PROP_DESC_END
@@ -123,24 +125,24 @@ participation_describe_props(Participation *participation)
 }
 
 static PropOffset participation_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  ORTHCONN_COMMON_PROPERTIES_OFFSETS,
   { "total", PROP_TYPE_BOOL, offsetof(Participation, total) },
   { NULL, 0, 0}
 };
 
 
 static void
-participation_get_props(Participation *participation, Property *props, guint nprops)
+participation_get_props(Participation *participation, GPtrArray *props)
 {
   object_get_props_from_offsets(&participation->orth.object, 
-                                participation_offsets, props, nprops);
+                                participation_offsets, props);
 }
 
 static void
-participation_set_props(Participation *participation, Property *props, guint nprops)
+participation_set_props(Participation *participation, GPtrArray *props)
 {
   object_set_props_from_offsets(&participation->orth.object, 
-                                participation_offsets, props, nprops);
+                                participation_offsets, props);
   participation_update_data(participation);
 }
 
@@ -154,33 +156,43 @@ participation_distance_from(Participation *participation, Point *point)
 
 static void
 participation_select(Participation *participation, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   orthconn_update_data(&participation->orth);
 }
 
-static void
+static ObjectChange*
 participation_move_handle(Participation *participation, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+			  Point *to, ConnectionPoint *cp,
+			  HandleMoveReason reason, ModifierKeys modifiers)
 {
+  ObjectChange *change;
   assert(participation!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
   
-  orthconn_move_handle(&participation->orth, handle, to, reason);
+  change = orthconn_move_handle(&participation->orth, handle, to, cp, 
+				reason, modifiers);
   participation_update_data(participation);
+
+  return change;
 }
 
-static void
+static ObjectChange*
 participation_move(Participation *participation, Point *to)
 {
-  orthconn_move(&participation->orth, to);
+  ObjectChange *change;
+
+  change = orthconn_move(&participation->orth, to);
   participation_update_data(participation);
+
+  return change;
 }
 
 static void
-participation_draw(Participation *participation, Renderer *renderer)
+participation_draw(Participation *participation, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   OrthConn *orth = &participation->orth;
   Point *points;
   Point *left_points;
@@ -194,10 +206,10 @@ participation_draw(Participation *participation, Renderer *renderer)
   last_left = 0.0;
   last_right = 0.0;
   
-  renderer->ops->set_linewidth(renderer, PARTICIPATION_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, PARTICIPATION_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   if (participation->total) {
     left_points = g_new0(Point, n);
@@ -249,12 +261,12 @@ participation_draw(Participation *participation, Renderer *renderer)
       right_points[i].y = points[i].y;
     }
     
-    renderer->ops->draw_polyline(renderer, left_points, n, &color_black);
-    renderer->ops->draw_polyline(renderer, right_points, n, &color_black);
+    renderer_ops->draw_polyline(renderer, left_points, n, &color_black);
+    renderer_ops->draw_polyline(renderer, right_points, n, &color_black);
     g_free(left_points);
     g_free(right_points);
   }  else {
-    renderer->ops->draw_polyline(renderer, points, n, &color_black);
+    renderer_ops->draw_polyline(renderer, points, n, &color_black);
   }
 }
 
@@ -262,7 +274,7 @@ static void
 participation_update_data(Participation *participation)
 {
   OrthConn *orth = &participation->orth;
-  OrthConnBBExtras *extra = &orth->extra_spacing;
+  PolyBBExtras *extra = &orth->extra_spacing;
   real extra_width;
   
   orthconn_update_data(orth);
@@ -392,6 +404,7 @@ participation_delete_segment_callback(Object *obj, Point *clicked, gpointer data
 static DiaMenuItem object_menu_items[] = {
   { N_("Add segment"), participation_add_segment_callback, NULL, 1 },
   { N_("Delete segment"), participation_delete_segment_callback, NULL, 1 },
+  ORTHCONN_COMMON_MENUS,
 };
 
 static DiaMenu object_menu = {
@@ -410,5 +423,6 @@ participation_get_object_menu(Participation *participation, Point *clickedpoint)
   /* Set entries sensitive/selected etc here */
   object_menu_items[0].active = orthconn_can_add_segment(orth, clickedpoint);
   object_menu_items[1].active = orthconn_can_delete_segment(orth, clickedpoint);
+  orthconn_update_object_menu(orth, clickedpoint, &object_menu_items[2]);
   return &object_menu;
 }
