@@ -19,19 +19,20 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "widgets.h"
@@ -52,7 +53,6 @@ typedef enum {
 } AnchorShape;
 
 typedef struct _Diamond Diamond;
-typedef struct _DiamondDefaultsDialog DiamondDefaultsDialog;
 
 struct _Diamond {
   Element element;
@@ -71,37 +71,22 @@ struct _Diamond {
 };
 
 typedef struct _DiamondProperties {
-  Color *fg_color;
-  Color *bg_color;
   gboolean show_background;
-  real border_width;
 
   real padding;
-  Color *font_color;
 } DiamondProperties;
 
-struct _DiamondDefaultsDialog {
-  GtkWidget *vbox;
-
-  GtkToggleButton *show_background;
-
-  GtkSpinButton *padding;
-  DiaFontSelector *font;
-  GtkSpinButton *font_size;
-};
-
-
-static DiamondDefaultsDialog *diamond_defaults_dialog;
 static DiamondProperties default_properties;
 
 static real diamond_distance_from(Diamond *diamond, Point *point);
 static void diamond_select(Diamond *diamond, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void diamond_move_handle(Diamond *diamond, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* diamond_move_handle(Diamond *diamond, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void diamond_move(Diamond *diamond, Point *to);
-static void diamond_draw(Diamond *diamond, Renderer *renderer);
+static ObjectChange* diamond_move(Diamond *diamond, Point *to);
+static void diamond_draw(Diamond *diamond, DiaRenderer *renderer);
 static void diamond_update_data(Diamond *diamond, AnchorShape h,AnchorShape v);
 static Object *diamond_create(Point *startpoint,
 			  void *user_data,
@@ -115,16 +100,14 @@ static void diamond_set_props(Diamond *diamond, GPtrArray *props);
 
 static void diamond_save(Diamond *diamond, ObjectNode obj_node, const char *filename);
 static Object *diamond_load(ObjectNode obj_node, int version, const char *filename);
-static GtkWidget *diamond_get_defaults(void);
-static void diamond_apply_defaults(void);
 
 static ObjectTypeOps diamond_type_ops =
 {
   (CreateFunc) diamond_create,
   (LoadFunc)   diamond_load,
   (SaveFunc)   diamond_save,
-  (GetDefaultsFunc)   diamond_get_defaults,
-  (ApplyDefaultsFunc) diamond_apply_defaults
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType diamond_type =
@@ -166,6 +149,7 @@ static PropDescription diamond_props[] = {
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
   PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT_ALIGNMENT,
   PROP_STD_SAVED_TEXT,
   
   { NULL, 0, 0, NULL, NULL, NULL, 0}
@@ -192,6 +176,7 @@ static PropOffset diamond_offsets[] = {
   {"text_font",PROP_TYPE_FONT,offsetof(Diamond,attrs.font)},
   {"text_height",PROP_TYPE_REAL,offsetof(Diamond,attrs.height)},
   {"text_colour",PROP_TYPE_COLOUR,offsetof(Diamond,attrs.color)},
+  {"text_alignment",PROP_TYPE_ENUM,offsetof(Diamond,attrs.alignment)},
   { NULL, 0, 0 },
 };
 
@@ -212,17 +197,6 @@ diamond_set_props(Diamond *diamond, GPtrArray *props)
   diamond_update_data(diamond,ANCHOR_MIDDLE,ANCHOR_MIDDLE);
 }
 
-static void
-diamond_apply_defaults()
-{
-  default_properties.show_background = gtk_toggle_button_get_active(diamond_defaults_dialog->show_background);
-
-  default_properties.padding = gtk_spin_button_get_value_as_float(diamond_defaults_dialog->padding);
-  attributes_set_default_font(
-      dia_font_selector_get_font(diamond_defaults_dialog->font),
-      gtk_spin_button_get_value_as_float(diamond_defaults_dialog->font_size));
-}
-
 static void
 init_default_values() {
   static int defaults_initialized = 0;
@@ -234,95 +208,6 @@ init_default_values() {
   }
 }
 
-static GtkWidget *
-diamond_get_defaults()
-{
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GtkWidget *checkdiamond;
-  GtkWidget *padding;
-  GtkWidget *fontsel;
-  GtkWidget *font_size;
-  GtkAdjustment *adj;
-  DiaFont *font;
-  real font_height;
-
-  if (diamond_defaults_dialog == NULL) {
-  
-    init_default_values();
-
-    diamond_defaults_dialog = g_new(DiamondDefaultsDialog, 1);
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    diamond_defaults_dialog->vbox = vbox;
-
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkdiamond = gtk_check_button_new_with_label(_("Draw background"));
-    diamond_defaults_dialog->show_background = GTK_TOGGLE_BUTTON( checkdiamond );
-    gtk_widget_show(checkdiamond);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkdiamond, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Text padding:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
-    padding = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(padding), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(padding), TRUE);
-    diamond_defaults_dialog->padding = GTK_SPIN_BUTTON(padding);
-    gtk_box_pack_start(GTK_BOX (hbox), padding, TRUE, TRUE, 0);
-    gtk_widget_show (padding);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    fontsel = dia_font_selector_new();
-    diamond_defaults_dialog->font = DIAFONTSELECTOR(fontsel);
-    gtk_box_pack_start (GTK_BOX (hbox), fontsel, TRUE, TRUE, 0);
-    gtk_widget_show (fontsel);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font size:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 1.0, 1.0);
-    font_size = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(font_size), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(font_size), TRUE);
-    diamond_defaults_dialog->font_size = GTK_SPIN_BUTTON(font_size);
-    gtk_box_pack_start(GTK_BOX (hbox), font_size, TRUE, TRUE, 0);
-    gtk_widget_show (font_size);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    gtk_widget_show (vbox);
-    gtk_widget_show (vbox);
-  }
-
-  gtk_toggle_button_set_active(diamond_defaults_dialog->show_background, 
-			       default_properties.show_background);
-
-  gtk_spin_button_set_value(diamond_defaults_dialog->padding,
-			    default_properties.padding);
-  attributes_get_default_font(&font, &font_height);
-  dia_font_selector_set_font(diamond_defaults_dialog->font, font);
-  gtk_spin_button_set_value(diamond_defaults_dialog->font_size, font_height);
-
-  return diamond_defaults_dialog->vbox;
-}
-
 static real
 diamond_distance_from(Diamond *diamond, Point *point)
 {
@@ -369,7 +254,7 @@ diamond_distance_from(Diamond *diamond, Point *point)
 
 static void
 diamond_select(Diamond *diamond, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   text_set_cursor(diamond->text, clicked_point, interactive_renderer);
   text_grab_focus(diamond->text, &diamond->element.object);
@@ -377,9 +262,10 @@ diamond_select(Diamond *diamond, Point *clicked_point,
   element_update_handles(&diamond->element);
 }
 
-static void
+static ObjectChange*
 diamond_move_handle(Diamond *diamond, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   AnchorShape horiz = ANCHOR_MIDDLE, vert = ANCHOR_MIDDLE;
 
@@ -387,7 +273,8 @@ diamond_move_handle(Diamond *diamond, Handle *handle,
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&diamond->element, handle->id, to, reason);
+  element_move_handle(&diamond->element, handle->id, to, cp, 
+		      reason, modifiers);
 
   switch (handle->id) {
   case HANDLE_RESIZE_NW:
@@ -410,19 +297,24 @@ diamond_move_handle(Diamond *diamond, Handle *handle,
     break;
   }
   diamond_update_data(diamond, horiz, vert);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 diamond_move(Diamond *diamond, Point *to)
 {
   diamond->element.corner = *to;
   
   diamond_update_data(diamond, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
+
+  return NULL;
 }
 
 static void
-diamond_draw(Diamond *diamond, Renderer *renderer)
+diamond_draw(Diamond *diamond, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point pts[4];
   Element *elem;
   
@@ -440,19 +332,19 @@ diamond_draw(Diamond *diamond, Renderer *renderer)
   pts[3].y += elem->height / 2.0;
 
   if (diamond->show_background) {
-    renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+    renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
   
-    renderer->ops->fill_polygon(renderer, 
+    renderer_ops->fill_polygon(renderer, 
 				pts, 4,
 				&diamond->inner_color);
   }
 
-  renderer->ops->set_linewidth(renderer, diamond->border_width);
-  renderer->ops->set_linestyle(renderer, diamond->line_style);
-  renderer->ops->set_dashlength(renderer, diamond->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linewidth(renderer, diamond->border_width);
+  renderer_ops->set_linestyle(renderer, diamond->line_style);
+  renderer_ops->set_dashlength(renderer, diamond->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
-  renderer->ops->draw_polygon(renderer, 
+  renderer_ops->draw_polygon(renderer, 
 			      pts, 4,
 			      &diamond->border_color);
 
@@ -490,6 +382,12 @@ diamond_update_data(Diamond *diamond, AnchorShape horiz, AnchorShape vert)
     if (grad > 4)     grad = 4;
     elem->width  = width  + height * grad;
     elem->height = height + width  / grad;
+  } else {
+    /* Need this for text alignment soon */
+    real grad = elem->width/elem->height;
+    if (grad < 1.0/4) grad = 1.0/4;
+    if (grad > 4)     grad = 4;
+    width = elem->width - height*grad;
   }
 
   /* move shape if necessary ... */
@@ -513,7 +411,17 @@ diamond_update_data(Diamond *diamond, AnchorShape horiz, AnchorShape vert)
   p = elem->corner;
   p.x += elem->width / 2.0;
   p.y += elem->height / 2.0 - diamond->text->height*diamond->text->numlines/2 +
-    font_ascent(diamond->text->font, diamond->text->height);
+      diamond->text->ascent;
+  switch (diamond->text->alignment) {
+  case ALIGN_LEFT:
+    p.x -= width/2;
+    break;
+  case ALIGN_RIGHT:
+    p.x += width/2;
+    break;
+  case ALIGN_CENTER:
+    break;
+  }
   text_set_position(diamond->text, &p);
 
   dw = elem->width / 8.0;
@@ -571,7 +479,7 @@ diamond_create(Point *startpoint,
   Object *obj;
   Point p;
   int i;
-  DiaFont *font;
+  DiaFont *font = NULL;
   real font_height;
 
   init_default_values();
@@ -586,7 +494,7 @@ diamond_create(Point *startpoint,
 
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
-  elem->height = DEFAULT_WIDTH;
+  elem->height = DEFAULT_HEIGHT;
 
   diamond->border_width =  attributes_get_default_linewidth();
   diamond->border_color = attributes_get_foreground();
@@ -603,7 +511,8 @@ diamond_create(Point *startpoint,
   diamond->text = new_text("", font, font_height, &p, &diamond->border_color,
 			   ALIGN_CENTER);
   text_get_attributes(diamond->text,&diamond->attrs);
-
+  dia_font_unref(font);
+  
   element_init(elem, 8, 16);
 
   for (i=0;i<16;i++) {
