@@ -21,14 +21,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "message.h"
@@ -41,7 +40,6 @@
 #define DEFAULT_BORDER 0.15
 
 typedef struct _Ellipse Ellipse;
-typedef struct _EllipseDefaultsDialog EllipseDefaultsDialog;
 
 struct _Ellipse {
   Element element;
@@ -56,33 +54,18 @@ struct _Ellipse {
   real dashlength;
 };
 
-
-typedef struct _EllipseProperties {
-  Color *fg_color;
-  Color *bg_color;
-  real border_width;
+static struct _EllipseProperties {
   gboolean show_background;
-  LineStyle line_style;
-  real dashlength;
-} EllipseProperties;
-
-struct _EllipseDefaultsDialog {
-  GtkWidget *vbox;
-
-  GtkToggleButton *show_background;
-};
-
- 
-static EllipseDefaultsDialog *ellipse_defaults_dialog;
-static EllipseProperties default_properties;
+} default_properties = { TRUE };
 
 static real ellipse_distance_from(Ellipse *ellipse, Point *point);
 static void ellipse_select(Ellipse *ellipse, Point *clicked_point,
-			   Renderer *interactive_renderer);
-static void ellipse_move_handle(Ellipse *ellipse, Handle *handle,
-				Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void ellipse_move(Ellipse *ellipse, Point *to);
-static void ellipse_draw(Ellipse *ellipse, Renderer *renderer);
+			   DiaRenderer *interactive_renderer);
+static ObjectChange* ellipse_move_handle(Ellipse *ellipse, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* ellipse_move(Ellipse *ellipse, Point *to);
+static void ellipse_draw(Ellipse *ellipse, DiaRenderer *renderer);
 static void ellipse_update_data(Ellipse *ellipse);
 static Object *ellipse_create(Point *startpoint,
 			      void *user_data,
@@ -97,16 +80,14 @@ static void ellipse_set_props(Ellipse *ellipse, GPtrArray *props);
 
 static void ellipse_save(Ellipse *ellipse, ObjectNode obj_node, const char *filename);
 static Object *ellipse_load(ObjectNode obj_node, int version, const char *filename);
-static GtkWidget *ellipse_get_defaults(void);
-static void ellipse_apply_defaults(void);
 
 static ObjectTypeOps ellipse_type_ops =
 {
   (CreateFunc) ellipse_create,
   (LoadFunc)   ellipse_load,
   (SaveFunc)   ellipse_save,
-  (GetDefaultsFunc)   ellipse_get_defaults,
-  (ApplyDefaultsFunc) ellipse_apply_defaults
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType ellipse_type =
@@ -180,97 +161,55 @@ ellipse_set_props(Ellipse *ellipse, GPtrArray *props)
   ellipse_update_data(ellipse);
 }
 
-static void
-ellipse_apply_defaults(void)
-{
-  default_properties.show_background = 
-    gtk_toggle_button_get_active(ellipse_defaults_dialog->show_background);
-}
- 
-static void
-init_default_values(void) {
-  static int defaults_initialized = 0;
-
-  if (!defaults_initialized) {
-    default_properties.show_background = 1;
-    defaults_initialized = 1;
-  }
-}
-
-static GtkWidget *
-ellipse_get_defaults(void)
-{
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *checkbox;
- 
-  if (ellipse_defaults_dialog == NULL) {
-
-    init_default_values();
- 
-    ellipse_defaults_dialog = g_new(EllipseDefaultsDialog, 1);
- 
-    vbox = gtk_vbox_new(FALSE, 5);
-    ellipse_defaults_dialog->vbox = vbox;
-
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Draw background"));
-    ellipse_defaults_dialog->show_background = GTK_TOGGLE_BUTTON( checkbox );
-    gtk_widget_show(checkbox);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
- 
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-    gtk_widget_show (vbox);
-    gtk_widget_show (vbox);
-  }
- 
-  gtk_toggle_button_set_active(ellipse_defaults_dialog->show_background, 
-			       default_properties.show_background);
- 
-  return ellipse_defaults_dialog->vbox;
-}
-
 static real
 ellipse_distance_from(Ellipse *ellipse, Point *point)
 {
-  Object *obj = &ellipse->element.object;
-  return distance_rectangle_point(&obj->bounding_box, point);
+  Element *elem = &ellipse->element;
+  Point center;
+
+  center.x = elem->corner.x+elem->width/2;
+  center.y = elem->corner.y+elem->height/2;
+
+  return distance_ellipse_point(&center, elem->width, elem->height,
+				ellipse->border_width, point);
 }
 
 static void
 ellipse_select(Ellipse *ellipse, Point *clicked_point,
-	       Renderer *interactive_renderer)
+	       DiaRenderer *interactive_renderer)
 {
   element_update_handles(&ellipse->element);
 }
 
-static void
+static ObjectChange*
 ellipse_move_handle(Ellipse *ellipse, Handle *handle,
-		    Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(ellipse!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
-  element_move_handle(&ellipse->element, handle->id, to, reason);
+  element_move_handle(&ellipse->element, handle->id, to, cp, reason, modifiers);
   ellipse_update_data(ellipse);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 ellipse_move(Ellipse *ellipse, Point *to)
 {
   ellipse->element.corner = *to;
   ellipse_update_data(ellipse);
+
+  return NULL;
 }
 
 static void
-ellipse_draw(Ellipse *ellipse, Renderer *renderer)
+ellipse_draw(Ellipse *ellipse, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point center;
   Element *elem;
   
@@ -283,19 +222,19 @@ ellipse_draw(Ellipse *ellipse, Renderer *renderer)
   center.y = elem->corner.y + elem->height/2;
 
   if (ellipse->show_background) {
-    renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+    renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
 
-    renderer->ops->fill_ellipse(renderer, 
+    renderer_ops->fill_ellipse(renderer, 
 				&center,
 				elem->width, elem->height,
 				&ellipse->inner_color);
   }
 
-  renderer->ops->set_linewidth(renderer, ellipse->border_width);
-  renderer->ops->set_linestyle(renderer, ellipse->line_style);
-  renderer->ops->set_dashlength(renderer, ellipse->dashlength);
+  renderer_ops->set_linewidth(renderer, ellipse->border_width);
+  renderer_ops->set_linestyle(renderer, ellipse->line_style);
+  renderer_ops->set_dashlength(renderer, ellipse->dashlength);
 
-  renderer->ops->draw_ellipse(renderer, 
+  renderer_ops->draw_ellipse(renderer, 
 			  &center,
 			  elem->width, elem->height,
 			  &ellipse->border_color);
@@ -334,6 +273,16 @@ ellipse_update_data(Ellipse *ellipse)
   ellipse->connections[7].pos.x = center.x + half_x;
   ellipse->connections[7].pos.y = center.y + half_y;
 
+  /* Update directions -- if the ellipse is very thin, these may not be good */
+  ellipse->connections[0].directions = DIR_NORTH|DIR_WEST;
+  ellipse->connections[1].directions = DIR_NORTH;
+  ellipse->connections[2].directions = DIR_NORTH|DIR_EAST;
+  ellipse->connections[3].directions = DIR_WEST;
+  ellipse->connections[4].directions = DIR_EAST;
+  ellipse->connections[5].directions = DIR_SOUTH|DIR_WEST;
+  ellipse->connections[6].directions = DIR_SOUTH;
+  ellipse->connections[7].directions = DIR_SOUTH|DIR_EAST;
+
   extra->border_trans = ellipse->border_width / 2.0;
   element_update_boundingbox(elem);
 
@@ -354,8 +303,6 @@ ellipse_create(Point *startpoint,
   Object *obj;
   int i;
 
-  init_default_values();
-
   ellipse = g_malloc0(sizeof(Ellipse));
   elem = &ellipse->element;
   obj = &elem->object;
@@ -366,14 +313,14 @@ ellipse_create(Point *startpoint,
 
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
-  elem->height = DEFAULT_WIDTH;
+  elem->height = DEFAULT_HEIGHT;
 
   ellipse->border_width =  attributes_get_default_linewidth();
   ellipse->border_color = attributes_get_foreground();
   ellipse->inner_color = attributes_get_background();
-  ellipse->show_background = default_properties.show_background;
   attributes_get_default_line_style(&ellipse->line_style,
 				    &ellipse->dashlength);
+  ellipse->show_background = default_properties.show_background;
 
   element_init(elem, 8, 8);
 
